
// Project: webCDwriter - Network CD Writing
// Author: Jrg Haeger

import java.io.*;

public class PutTracks extends PutFiles
{
	public PutTracks(DirTreeNode CDRootNode, int sessionSize)
	{
		super(CDRootNode, sessionSize);
	}

	protected void putFile(String dirName, DirTreeNode node)
		throws java.io.IOException, CommandError
	{
		File file = node.getFile();
		String fileName = dirName + "/" + node.toString();
		Version.showDebugInfo(1, fileName + " " + file);
		view.setMessage(fileName);

		String trackType = "image";
		if (file.getName().toLowerCase().endsWith(".wav"))
			trackType = "wav";
		else
		if (file.getName().toLowerCase().endsWith(".mp3"))
			trackType = "mp3";

		if (file instanceof ServerFile)
		{
			outStream.writeBytes("linkTrack "
				+ spaceToEsc(file.getPath()) + " "
				+ spaceToEsc(trackType) + "\n");
			readResult();
			if (!lastResultOK())
				throw new CommandError();

			if (file.length() > 0)
				bytesTransmitted +=
					1024 * ((file.length() - 1) / 1024 + 1);
			view.setProgress(bytesTransmitted);
			return;
		}

		if (MainWin.isWAV(file))
			trackType = "wav";

		FileInputStream inFile;
		try {
			inFile = new FileInputStream(file);
		}
		catch (java.io.IOException e) {
			System.out.println("can't read " + file);
			return;
		}

		while (!cancel)
		{
			byte bs[] = new byte[16*1024];
			int len = inFile.read(bs);
			if (len == -1)
				break;
			outStream.writeBytes("putTrack "
				+ trackType + " " + len
				+ "\n");
			outStream.write(bs, 0, len);
			bytesTransmitted += len;
			if (len % 1024 > 0)
				bytesTransmitted += 1024 - len % 1024;
			view.setProgress(bytesTransmitted);

			readResult();
			if (!lastResultOK())
				throw new CommandError();

			trackType = "continue";
		}
	}
}
